﻿using UnityEngine;
using System.Threading;
using System.Collections;
using System.Collections.Generic;

// This script handles the necessary simulation functions and threading behaviors
public class Simulation : MonoBehaviour {
	
	public static readonly float timeStep = 0.01f;
	
	// TODO: Some other event should ignite the starting tile and set this to be true
	public static volatile bool simRunning = true;
	private static readonly AutoResetEvent simSync = new AutoResetEvent(false);

	private static HashSet <StateChangeEvent> [] visDBuf = new HashSet<StateChangeEvent> [2];
	private static HashSet <StateChangeEvent> [] simDBuf = new HashSet<StateChangeEvent> [2];

	// Both the UI and Worker threads will be writing to the HashSet indicated by this index
	// and will be reading from the other HashSet.
	// This index is switched during the synchronizeSimulation Coroutine
	private static volatile int bufIndex = 0;

	// NOTE: The following three fields are used by the worker thread to query the terrain data.
	// The terrain data cannot be accessed by other threads, so necessary information has been copied.

	private Thread simThread = null;

	void Start () {
		TerrainManager.initTerrainManager();

		visDBuf [0] = new HashSet <StateChangeEvent> ();
		visDBuf [1] = new HashSet <StateChangeEvent> ();

		simDBuf [0] = new HashSet <StateChangeEvent> ();
		simDBuf [1] = new HashSet <StateChangeEvent> ();

		simThread = new Thread(Simulation.runTimeStep);
		simThread.Start();

		//SpaceTree.getTile(new CanAddr()).curTemp[0] = 110f;
		//SpaceTree.getTile(new CanAddr()).curTemp[1] = 110f;

		StartCoroutine (synchronizeSimulation());
	}
	
	IEnumerator synchronizeSimulation () {
		while (simRunning) {
			bufIndex = (bufIndex + 1) % 2;
			simDBuf[bufIndex].Clear();

			simSync.Set();

			if (visDBuf [(bufIndex+1) %2].Count == 0) {
				yield return new WaitForSeconds (timeStep);
			} else {
				float visChangeStep = timeStep / visDBuf [(bufIndex+1) % 2].Count;
				
				foreach (StateChangeEvent vbEvent in visDBuf [(bufIndex+1) % 2]) {
					// TODO: This code is temporary, just for visualization

					if (vbEvent.state == Tile.FIRE_MASK) {
						Vector2 tilePos = TerrainManager.convertSimToTerrain(LatAddr.convertLatAddrToVector(CanAddr.convertCanAddrToLatAddr(vbEvent.tileAddr)));
						float [,,] map = new float[4,4,3];

						for (int i = 0; i < 4; i++) {
							for (int j = 0; j < 4; j++) {
								map[i, j, 0] = 0;
								map[i, j, 1] = 1;
								map[i, j, 2] = 0;
							}
						}

						Terrain.activeTerrain.terrainData.SetAlphamaps(Mathf.RoundToInt(tilePos.x) - 2, Mathf.RoundToInt(tilePos.y) - 2, map);

					}
					else {
						Vector2 tilePos =TerrainManager. convertSimToTerrain(LatAddr.convertLatAddrToVector(CanAddr.convertCanAddrToLatAddr(vbEvent.tileAddr)));
						float [,,] map = new float[4,4,3];
						
						for (int i = 0; i < 4; i++) {
							for (int j = 0; j < 4; j++) {
								map[i, j, 0] = 0;
								map[i, j, 1] = 0;
								map[i, j, 2] = 1;
							}
						}
						
						Terrain.activeTerrain.terrainData.SetAlphamaps(Mathf.RoundToInt(tilePos.x) - 2, Mathf.RoundToInt(tilePos.y) - 2, map);
					}
					
					yield return new WaitForSeconds (visChangeStep);
				}
			}
		}
	}

	private static void runTimeStep () {
		while (simRunning) {
			simSync.WaitOne();
			visDBuf[bufIndex].Clear();

			// TODO: Apply all SimBoundEvents from simDBuf[(bufIndex+1) %2]
			// TODO: Run one time step of the simulation
			SpaceTree.updateState();
			SpaceTree.updateProperties();
		}
	}

	public static void postVisBoundChange (StateChangeEvent scEvnt) {
		visDBuf[bufIndex].Add(scEvnt);
	}
}